@@ -6,8 +6,10 @@ import os
 import shutil
 import time
 import pickle
+import re
+import json
 
-from django.utils.six.moves.urllib.parse import urljoin
+from urllib.parse import urljoin
 from django.core.management.base import BaseCommand
 from django.contrib.staticfiles import finders
 from django.conf import settings
@@ -15,8 +17,10 @@ from django.templatetags.static import PrefixNode
 from django.apps import apps
 
 from .npm_install import install_npm
+from .collectstatic import Command as CSCommand
 from npm_mjs import signals
 
+
 if settings.PROJECT_PATH:
     PROJECT_PATH = settings.PROJECT_PATH
 else:
@@ -33,7 +37,7 @@ TRANSPILE_TIME_PATH = os.path.join(
 )
 
 LAST_RUN = {
-    'version': 0
+    'transpile': 0
 }
 
 try:
@@ -41,10 +45,8 @@ try:
         TRANSPILE_TIME_PATH,
         'rb'
     ) as f:
-        LAST_RUN['version'] = pickle.load(f)
-except EOFError:
-    pass
-except IOError:
+        LAST_RUN = {**LAST_RUN, **pickle.load(f)}
+except (EOFError, IOError, TypeError):
     pass
 
 WEBPACK_CONFIG_JS_PATH = os.path.join(
@@ -81,7 +83,7 @@ class Command(BaseCommand):
         else:
             force = False
         start = int(round(time.time()))
-        npm_install = install_npm()
+        npm_install = install_npm(force)
         self.stdout.write("Transpiling...")
         js_paths = finders.find('js/', True)
         # Remove paths inside of collection dir
@@ -115,16 +117,25 @@ class Command(BaseCommand):
                 return
             # Remove any previously created static output dirs
             shutil.rmtree(transpile_path)
-        LAST_RUN['version'] = start
         if not os.path.exists(TRANSPILE_CACHE_PATH):
             os.makedirs(TRANSPILE_CACHE_PATH)
+        # We reload the file as other values may have changed in the meantime
+        try:
+            with open(
+                TRANSPILE_TIME_PATH,
+                'rb'
+            ) as f:
+                LAST_RUN = {**LAST_RUN, **pickle.load(f)}
+        except (EOFError, IOError, TypeError):
+            pass
+        LAST_RUN['transpile'] = start
         with open(
             TRANSPILE_TIME_PATH,
             'wb'
         ) as f:
-            pickle.dump(LAST_RUN['version'], f)
+            pickle.dump(LAST_RUN, f)
         # Create a static output dir
-        out_dir = os.path.join(transpile_path, "js/transpile/")
+        out_dir = os.path.join(transpile_path, "js/")
         os.makedirs(out_dir)
         with open(os.path.join(transpile_path, "README.txt"), 'w') as f:
             f.write(
@@ -235,70 +246,76 @@ class Command(BaseCommand):
             static_base_url = staticfiles_storage.base_url
         else:
             static_base_url = PrefixNode.handle_simple("STATIC_URL")
-        transpile_base_url = urljoin(static_base_url, 'js/transpile/')
-
-        webpack_config_js = 'var webpack = require("webpack");'
-        webpack_config_js += 'module.exports = {\n'
-        if settings.DEBUG:
-            mode = 'development'
+        transpile_base_url = urljoin(static_base_url, 'js/')
+        if (
+            hasattr(settings, 'WEBPACK_CONFIG_TEMPLATE') and
+            settings.WEBPACK_CONFIG_TEMPLATE
+        ):
+            webpack_config_template_path = settings.WEBPACK_CONFIG_TEMPLATE
         else:
-            mode = 'production'
-        webpack_config_js += ' mode: "{}",\n'.format(mode)
-        webpack_config_js += ' module: {\n'
-        webpack_config_js += '  rules: [\n'
-        webpack_config_js += '   {\n'
-        webpack_config_js += '    test: /\\.(js|mjs)$/,\n'
-        if settings.DEBUG:
-            webpack_config_js += '    exclude: /node_modules/,\n'
-        webpack_config_js += '    use: {\n'
-        webpack_config_js += '     loader: "babel-loader",\n'
-        webpack_config_js += '     options: {\n'
-        webpack_config_js += '      presets: [\n'
-        webpack_config_js += '       "@babel/preset-env"\n'
-        webpack_config_js += '      ],\n'
-        webpack_config_js += '      plugins: [\n'
-        webpack_config_js += '       "@babel/plugin-syntax-dynamic-import"\n'
-        webpack_config_js += '      ]\n'
-        webpack_config_js += '     }\n'
-        webpack_config_js += '    }\n'
-        webpack_config_js += '   }\n'
-        webpack_config_js += '  ]\n'
-        webpack_config_js += ' },\n'
-        webpack_config_js += ' output: {\n'
-        webpack_config_js += '  path: "{}",\n'.format(out_dir)
-        webpack_config_js += '  chunkFilename: "{}-[id].js",\n'.format(
-            LAST_RUN['version']
-        )
-        webpack_config_js += '  publicPath: "{}",\n'.format(
-            transpile_base_url
-        )
-        webpack_config_js += ' },\n'
-        webpack_config_js += ' plugins: [\n'
-        webpack_config_js += '  new webpack.DefinePlugin({'
-        webpack_config_js += '   "process.env.TRANSPILE_VERSION": process.env.TRANSPILE_VERSION\n'
-        webpack_config_js += '  })'
-        webpack_config_js += ' ],\n'
-        webpack_config_js += ' entry: {\n'
+            webpack_config_template_path = os.path.join(
+                os.path.dirname(
+                    os.path.realpath(__file__)
+                ),
+                'webpack.config.template.js'
+            )
+        entries = {}
         for mainfile in mainfiles:
             basename = os.path.basename(mainfile)
             modulename = basename.split('.')[0]
             file_path = os.path.join(cache_path, basename)
-            webpack_config_js += '  {}: "{}",\n'.format(
-                modulename,
-                file_path
+            entries[modulename] = file_path
+        find_static = CSCommand()
+        find_static.set_options(**{
+            'interactive': False,
+            'verbosity': 0,
+            'link': False,
+            'clear': False,
+            'dry_run': True,
+            'ignore_patterns': ['js/', 'admin/'],
+            'use_default_ignore_patterns': True,
+            'post_process': True
+        })
+        found_files = find_static.collect()
+        static_frontend_files = (
+            found_files['modified'] +
+            found_files['unmodified'] +
+            found_files['post_processed']
+        )
+        transpile = {
+            'OUT_DIR': out_dir,
+            'VERSION': LAST_RUN['transpile'],
+            'BASE_URL': transpile_base_url,
+            'ENTRIES': entries,
+            'STATIC_FRONTEND_FILES': list(map(
+                lambda x: urljoin(static_base_url, x),
+                static_frontend_files
+            ))
+        }
+        with open(webpack_config_template_path, 'r') as f:
+            webpack_config_template = f.read()
+        settings_dict = {}
+        for var in dir(settings):
+            if var in ['DATABASES', 'SECRET_KEY']:
+                # For extra security, we do not copy DATABASES or SECRET_KEY
+                continue
+            settings_dict[var] = getattr(settings, var)
+        webpack_config_js = \
+            webpack_config_template.replace(
+                'window.transpile', json.dumps(transpile)
+            ).replace(
+                'window.settings', json.dumps(
+                    settings_dict,
+                    default=lambda x: False
+                )
             )
-        webpack_config_js += ' }\n'
-        webpack_config_js += '}\n'
 
         if webpack_config_js is not OLD_WEBPACK_CONFIG_JS:
             with open(WEBPACK_CONFIG_JS_PATH, 'w') as f:
                 f.write(webpack_config_js)
-        env = os.environ.copy()
-        env['TRANSPILE_VERSION'] = str(LAST_RUN['version'])
         call(
             ['./node_modules/.bin/webpack'],
-            cwd=TRANSPILE_CACHE_PATH,
-            env=env
+            cwd=TRANSPILE_CACHE_PATH
         )
         end = int(round(time.time()))
         self.stdout.write(
